@@ -1,9 +1,11 @@
 #' @title Compile a PRQL query into a SQL query
 #' @param prql_query a PRQL query string.
-#' @param target a compile target name to use. If not specified, the target contained in the query will be used.
+#' @param target a compile target name to use. If not specified (`NULL`),
+#' the target contained in the query will be used.
 #' All available target names can be listed with the [prql_get_targets] function.
-#' @param format a logical flag. Whether to format the SQL query.
-#' @param signature_comment a logical flag. Whether to add a signature comment to the output SQL query.
+#' @param format a logical flag (default: `TRUE`). Whether to format the SQL query.
+#' @param signature_comment a logical flag. (default: `TRUE`).
+#' Whether to add a signature comment to the output SQL query.
 #' @return a SQL query string
 #' @seealso [prql_get_targets]
 #' @examples
@@ -21,7 +23,7 @@
 #'   prql_compile("sql.duckdb") |>
 #'   cat()
 #'
-#' # If the `target` argument is `NULL` (default) or `NA` or `"sql.any"`,
+#' # If the `target` argument is `NULL` (default) or `"sql.any"`,
 #' # the target specified in the header of the query will be used.
 #' "
 #' prql target:sql.duckdb
@@ -38,12 +40,11 @@ prql_compile <- function(
     target = getOption("prqlr.target", default = NULL),
     format = getOption("prqlr.format", default = TRUE),
     signature_comment = getOption("prqlr.signature_comment", default = TRUE)) {
-  compile(prql_query, target, format, signature_comment) |>
-    unwrap()
+  compile(prql_query, target %||% "sql.any", format, signature_comment)
 }
 
-#' @title prql-compiler's version
-#' @return a [numeric_version] with the version of the built-in prql-compiler.
+#' @title prqlc's version
+#' @return a [numeric_version] with the version of the built-in prqlc.
 #' @examples
 #' prql_version()
 #' @export
