@@ -12,10 +12,16 @@
 
 use crate::dialect::Dialect;
 
+/// A permissive, general purpose [`Dialect`], which parses a wide variety of SQL
+/// statements, from many different dialects.
 #[derive(Debug, Default)]
 pub struct GenericDialect;
 
 impl Dialect for GenericDialect {
+    fn is_delimited_identifier_start(&self, ch: char) -> bool {
+        ch == '"' || ch == '`'
+    }
+
     fn is_identifier_start(&self, ch: char) -> bool {
         ch.is_alphabetic() || ch == '_' || ch == '#' || ch == '@'
     }
@@ -32,4 +38,8 @@ impl Dialect for GenericDialect {
     fn supports_group_by_expr(&self) -> bool {
         true
     }
+
+    fn supports_start_transaction_modifier(&self) -> bool {
+        true
+    }
 }
