@@ -91,7 +91,7 @@ pub fn parse_create_stage(
 ) -> Result<Statement, ParserError> {
     //[ IF NOT EXISTS ]
     let if_not_exists = parser.parse_keywords(&[Keyword::IF, Keyword::NOT, Keyword::EXISTS]);
-    let name = parser.parse_object_name()?;
+    let name = parser.parse_object_name(false)?;
     let mut directory_table_params = Vec::new();
     let mut file_format = Vec::new();
     let mut copy_options = Vec::new();
@@ -181,7 +181,7 @@ pub fn parse_snowflake_stage_name(parser: &mut Parser) -> Result<ObjectName, Par
         }
         _ => {
             parser.prev_token();
-            Ok(parser.parse_object_name()?)
+            Ok(parser.parse_object_name(false)?)
         }
     }
 }
@@ -219,7 +219,7 @@ pub fn parse_copy_into(parser: &mut Parser) -> Result<Statement, ParserError> {
         }
         _ => {
             parser.prev_token();
-            from_stage = parser.parse_object_name()?;
+            from_stage = parser.parse_object_name(false)?;
             stage_params = parse_stage_params(parser)?;
 
             // as
