@@ -333,9 +333,19 @@ where
         .map(ExprKind::Internal)
         .map_with_span(into_expr);
 
+    let generic_args = ident_part()
+        .then_ignore(ctrl(':'))
+        .then(type_expr().separated_by(ctrl('|')))
+        .map(|(name, domain)| GenericTypeParam { name, domain })
+        .separated_by(ctrl(','))
+        .at_least(1)
+        .delimited_by(ctrl('<'), ctrl('>'))
+        .or_not()
+        .map(|x| x.unwrap_or_default());
+
     choice((
         // func
-        keyword("func").ignore_then(
+        keyword("func").ignore_then(generic_args).then(
             param
                 .clone()
                 .separated_by(new_line().repeated())
@@ -343,14 +353,14 @@ where
                 .allow_trailing(),
         ),
         // plain
-        param.repeated(),
+        param.repeated().map(|params| (Vec::new(), params)),
     ))
     .then_ignore(just(Token::ArrowThin))
     // return type
     .then(type_expr().delimited_by(ctrl('<'), ctrl('>')).or_not())
     // body
     .then(choice((internal, func_call(expr))))
-    .map(|((params, return_ty), body)| {
+    .map(|(((generic_type_params, params), return_ty), body)| {
         let (pos, name) = params
             .into_iter()
             .map(|((name, ty), default_value)| FuncParam {
@@ -366,6 +376,7 @@ where
 
             body: Box::new(body),
             return_ty,
+            generic_type_params,
         })
     })
     .map(ExprKind::Func)
